from django.contrib import admin
from rolodex.models import OpenRecordsLaw,PersonRole,P2P_Type,Org2Org_Type,P2Org_Type,Tag,Contact,Person,Org,OrgContactRole



class ContactInline(admin.StackedInline):
	model = Contact
	extra = 0

class POAdmin(admin.ModelAdmin):
	inlines = [ContactInline,]

admin.site.register(OpenRecordsLaw)
admin.site.register(PersonRole)
admin.site.register(OrgContactRole)
admin.site.register(P2P_Type)
admin.site.register(Org2Org_Type)
admin.site.register(P2Org_Type)
admin.site.register(Tag)

admin.site.register(Person,POAdmin)
admin.site.register(Org,POAdmin)


from rest_framework import viewsets
from rest_framework.decorators import api_view
from rolodex.API import serializers
from rolodex.models import Person,Org,Contact,PersonRole,OrgContactRole,P2P,Org2Org,P2Org,Org2P,P2P_Type,Org2Org_Type,P2Org_Type
from rest_framework import permissions
from rest_framework.views import APIView
from rest_framework.response import Response
from django.shortcuts import get_object_or_404

class RoleViewSet(viewsets.ModelViewSet):
	queryset = PersonRole.objects.all()
	serializer_class = serializers.RoleSerializer

class ContactRoleViewSet(viewsets.ModelViewSet):
	queryset = OrgContactRole.objects.all()
	serializer_class = serializers.ContactRoleSerializer


class P2P_TypeViewSet(viewsets.ModelViewSet):
	queryset = P2P_Type.objects.all()
	serializer_class = serializers.P2P_TypeSerializer
class Org2Org_TypeViewSet(viewsets.ModelViewSet):
	queryset = Org2Org_Type.objects.all()
	serializer_class = serializers.Org2Org_TypeSerializer
class P2Org_TypeViewSet(viewsets.ModelViewSet):
	queryset = P2Org_Type.objects.all()
	serializer_class = serializers.P2Org_TypeSerializer


class PersonViewSet(viewsets.ModelViewSet):
    queryset = Person.objects.all().order_by('id')
    serializer_class = serializers.PersonSerializer

class OrgViewSet(viewsets.ModelViewSet):
	queryset = Org.objects.all().order_by('id')
	serializer_class = serializers.OrgSerializer

class ContactViewSet(viewsets.ModelViewSet):
	queryset = Contact.objects.all().order_by('id')
	serializer_class = serializers.ContactSerializer

class P2PViewSet(viewsets.ModelViewSet):
	queryset = P2P.objects.all().order_by('id')
	serializer_class = serializers.P2PSerializer

class Org2OrgViewSet(viewsets.ModelViewSet):
	queryset = Org2Org.objects.all().order_by('id')
	serializer_class = serializers.Org2OrgSerializer

class Org2PViewSet(viewsets.ModelViewSet):
	queryset = Org2P.objects.all().order_by('id')
	serializer_class = serializers.Org2PSerializer

class P2OrgViewSet(viewsets.ModelViewSet):
	queryset = P2Org.objects.all().order_by('id')
	serializer_class = serializers.P2OrgSerializer


class GetEmployees(APIView):
	model = Person
	queryset = Person.objects.all()
	'''
	Get all an orgs empoyees.
	'''
	def get(self,request,pk):
		org = get_object_or_404(Org,pk=pk)
		employees = org.get_employees()
		serializer = serializers.PersonSerializer(employees,many=True,context={'request': request})
		return Response(serializer.data)




#######################
## DATABASE SETTINGS ##
#######################
import dj_database_url
from django.conf import settings
import os

if not settings.DATABASES.has_key('rolodex'):
	if os.environ.has_key('ROLODEX_DB'):
	    settings.DATABASES['rolodex'] = dj_database_url.parse(os.environ.get('ROLODEX_DB'))
	    settings.DATABASE_ROUTERS.append('rolodex.routers.RolodexRouter')
else:
	settings.DATABASE_ROUTERS.append('rolodex.routers.RolodexRouter')
#######################################################

if not hasattr(settings, 'ROLODEX_SECURE'):
	settings.ROLODEX_SECURE = True

settings.TEMPLATE_CONTEXT_PROCESSORS += ('rolodex.context_processors.modal_context',)

###################
## Custom Config ##
###################

from django.apps import AppConfig

class RolodexConfig(AppConfig):
	name = 'rolodex'
	verbose_name='Rolodex'


from rest_framework import routers

from rolodex.API import views

router = routers.DefaultRouter()
router.register(r'roles', views.RoleViewSet)
router.register(r'contact-roles', views.ContactRoleViewSet)
router.register(r'contacts', views.ContactViewSet)
router.register(r'people', views.PersonViewSet)
router.register(r'orgs', views.OrgViewSet)
router.register(r'relationships/person_to_person', views.P2PViewSet)
router.register(r'relationships/org_to_org', views.Org2OrgViewSet)
router.register(r'relationships/org_to_person', views.Org2PViewSet)
router.register(r'relationships/person_to_org', views.P2OrgViewSet)
router.register(r'relationships/types/person_to_person',views.P2P_TypeViewSet)
router.register(r'relationships/types/org_to_org',views.Org2Org_TypeViewSet)
router.register(r'relationships/types/person_to_org',views.P2Org_TypeViewSet)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations

from django.conf import settings

def initial_data(apps, schema_editor):
	if settings.DATABASES.has_key('rolodex'):
		if not schema_editor.connection.alias == 'rolodex':
			return
	else:
		if not schema_editor.connection.alias == 'default':
			return
	P2Org_Type = apps.get_model('rolodex','P2Org_Type')
	P2Org_Type.objects.get_or_create(slug='employment',relationship_type='employment')

	OrgContactRole = apps.get_model('rolodex','OrgContactRole')
	OrgContactRole.objects.get_or_create(slug='public-records-contact',role='public records contact',description="A contact to receive public records requests.")

	PersonRole = apps.get_model('rolodex','PersonRole')
	PersonRole.objects.get_or_create(slug='public-information-officer',role='public information officer', description="A person responsible for fielding public records requests.")

class Migration(migrations.Migration):

	dependencies = [
		('rolodex', '0001_initial'),
	]

	operations = [
		migrations.RunPython(initial_data),
	]



from django.test import TestCase, RequestFactory
from django.test import Client
from django.core.urlresolvers import reverse
from rolodex.models import *
from django_webtest import WebTest
import pdb

client = Client()

class RolodexModelsTestCase(TestCase):

	print("Starting models tests...")

	fixtures = ['rolodex_models_testdata.json']
	
	def setUp(self):
		self.o1 = Org.objects.get(slug="acme-corp")
		self.p1 = Person.objects.get(slug="john-doe")
		self.o2 = Org.objects.get(slug="ajax-corp")
		self.p2 = Person.objects.get(slug="jane-doe")

	def test_getter_methods(self):
		self.assertEqual(list(self.p1.get_relations()['orgs']),[self.o1])
		self.assertEqual(list(self.p1.get_employer()),[self.o1])
		self.assertEqual(list(self.o1.get_employees()),[self.p1])
		self.assertEqual(list(self.o1.get_employees_by_role('boss')),[self.p1])
		self.assertEqual(self.o1.get_relations_by_type('employment')['people'],[{'type':'employment','relation':self.p1}])
		self.assertEqual(self.o1.get_relations_with_type()['people'],[{'type':'employment','relation':self.p1}] )
		self.assertEqual(self.p1.nx_graph().has_node(self.o1),True)
		print(" >> Passed getter methods")
	
	def test_create_relationships(self):
		self.p1.add_p2p(self.p2)
		self.assertEqual(list(self.p1.get_relations()['people']),[self.p2])
		
		self.o1.add_org2org(self.o2,**{'hierarchy':'parent'})
		self.assertEqual(Org2Org.objects.get(from_ent=self.o2,to_ent=self.o1).hierarchy,'child')

		self.o1.add_org2p(self.p2)
		self.assertEqual(list(self.p2.get_relations()['orgs']),[self.o1])
		print(" >> Passed relationship create methods")

	def test_delete_relationships(self):
		self.p1.remove_p2p(self.p2)
		self.assertEqual(list(self.p2.get_relations_with_type()['people']),[])

		self.o1.remove_org2org(self.o2)
		self.assertEqual(list(self.o2.get_relations()['orgs']),[])
		
		self.o1.remove_org2p(self.p2)
		self.assertEqual(list(self.p2.get_relations()['orgs']),[])
		print(" >> Passed relationship delete methods")


	'''
	This test is not passing right now. See Org2Org model save method.
	'''
	# def test_hierarchy_recursion(self):
	# 	self.o1.add_org2org(self.o2)
	# 	relation = Org2Org.objects.get(from_ent=self.o1,to_ent=self.o2)
	# 	relation.hierarchy = 'parent'
	# 	relation.save()
	# 	self.assertEqual(Org2Org.objects.get(from_ent=self.o2,to_ent=self.o1).hierarchy,'child')
	# 	print(" >> Passed hierarchy recursion")

class RolodexViewsTestCase(WebTest):

	print("Starting views tests...")

	fixtures = ['rolodex_views_testdata.json']
	
	def setUp(self):
		self.o1 = Org.objects.get(slug="acme-corp")
		self.p1 = Person.objects.get(slug="john-doe")
		self.o2 = Org.objects.get(slug="ajax-corp")
		self.p2 = Person.objects.get(slug="jane-doe")

	def test_home(self):	
		response = client.get(reverse('rolodex_home'))
		self.assertEqual(response.status_code,200)
		print(" >> Passed home")
	
	def test_search(self):
		response = client.get(reverse('rolodex_org',args=[self.o1.slug]))
		self.assertContains(response, self.o1.orgName , status_code=200)
		response = client.get(reverse('rolodex_person',args=[self.p1.slug]))
		self.assertContains(response, self.p1.lastName , status_code=200)
		print(" >> Passed entity pages")

	def test_create_edit(self):
		'''
		Add a person with a bad email address. Fix the email and check for successful resubmit.
		'''
		form = self.app.get(reverse('rolodex_new_person',args=[self.o2.slug])).form
		form['lastName'] = 'Rabbit'
		form['firstName'] = 'Roger'
		form['person_contact-0-contact'] = 'nobugsbunny'
		form['person_contact-0-type'] = 'email'
		print(form['lastName'].value)
		response = form.submit()
		self.assertContains(response, 'Enter a valid email address.')
		form = response.form
		form['person_contact-0-contact'] = 'nobugs@gmail.com'
		response = form.submit()
		self.assertEqual(response.context['success'], True )
		'''
		Edit Person
		'''
		form = self.app.get(reverse('rolodex_edit_person',args=[self.p2.slug])).form
		form['lastName'] = 'Doe, Jr.'
		form['position'] = 'Scientist'
		response = form.submit().follow()
		self.assertContains(response, 'Doe, Jr.' , status_code=200)
		'''
		Edit Org
		'''
		form = self.app.get(reverse('rolodex_edit_org',args=[self.o2.slug])).form
		form['orgName'] = 'AJAX, Inc.'
		response = form.submit().follow()
		self.assertContains(response, 'AJAX, Inc.' , status_code=200)
		'''
		Edit Contact
		'''
		form = self.app.get(reverse('rolodex_edit_person',args=['roger-rabbit'])).form
		form['person_contact-0-contact'] = 'theRealRoger@gmail.com'
		response= form.submit().follow()
		self.assertContains(response, 'theRealRoger@gmail.com')
		#Delete the contact
		form = self.app.get(reverse('rolodex_edit_person',args=['roger-rabbit'])).form
		form['person_contact-0-DELETE'] = True
		response = form.submit().follow()
		self.assertNotContains(response, 'nobugs@gmail.com' , status_code=200)
		print(" >> Passed entity create and edit pages")

	def test_relation_create(self):
		'''
		Correspond to the new_org_relation & new_person_relation views.

		Org2Org
		'''
		form = self.app.get(reverse('rolodex_new_org_relation',args=[self.o1.slug])).forms[0]
		form['to_ent'] = self.o2.pk
		form['hierarchy'] = 'parent'
		response = form.submit()
		self.assertEqual(response.status_code,200)
		self.assertEqual(Org2Org.objects.get(from_ent=self.o2,to_ent=self.o1).hierarchy,'child')
		'''
		Check no duplicate relationships
		'''
		form = self.app.get(reverse('rolodex_new_org_relation',args=[self.o1.slug])).forms[0]
		form['to_ent'] = self.o2.pk
		response = form.submit()
		self.assertContains(response, 'That relationship already exists.')

		'''
		Org2P
		'''
		form = self.app.get(reverse('rolodex_new_org_relation',args=[self.o2.slug])).forms[1]
		form['to_ent'] = self.p2.pk
		response = form.submit()
		self.assertEqual(response.context['saved'],True)
		'''
		P2P
		'''
		form = self.app.get(reverse('rolodex_new_person_relation',args=[self.p2.slug])).forms[1]
		form['to_ent'] = self.p1.pk
		response = form.submit()
		self.assertEqual(response.context['saved'],True)
		'''
		P2Org
		'''
		form = self.app.get(reverse('rolodex_new_person_relation',args=[self.p2.slug])).forms[0]
		form['to_ent'] = self.o1.pk
		response = form.submit()
		self.assertEqual(response.context['saved'],True)

		print(" >> Passed relationship create page")

	def test_delete(self):
		response = client.post(reverse('rolodex_delete_person',args=[self.p2.slug]))
		self.assertEqual(response.status_code , 302)
		response = client.post(reverse('rolodex_delete_org',args=[self.o2.slug]))
		self.assertEqual(response.status_code , 302)
		print(" >> Passed delete page")



'''
Fixtures...
'''
# from rolodex.models import *
# RELATION, get = P2Org_Type.objects.get_or_create(relationship_type='employment')
# ROLE = PersonRole.objects.create(role="boss")
# CONTACT_ROLE = OrgContactRole.objects.create(role="front desk")
# o1 = Org.objects.create(orgName="ACME, Corp.")
# p1 = Person.objects.create(firstName='John', lastName='Doe', role=ROLE)
# p1.add_p2org(o1,**{'relation':RELATION})
# Org.objects.create(orgName="AJAX, Corp.")
# Person.objects.create(firstName='Jane', lastName='Doe')

#python manage.py dumpdata rolodex --format=json --indent=4 > testproject/fixtures/rolodex_models_testdata.json
#python manage.py dumpdata rolodex --format=json --indent=4 > testproject/fixtures/rolodex_views_testdata.json


from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import redirect, render, render_to_response, HttpResponseRedirect,get_object_or_404
from django.template import RequestContext
from django.http import HttpResponse
from rolodex.models import Tag,Org,Person,P2P,Org2Org,P2Org,Org2P,P2Org_Type,P2P_Type,Org2Org_Type
from rolodex.forms import OrgForm,OrgFormSet,PersonForm,PersonFormSet,P2PForm,Org2OrgForm,P2OrgForm,Org2PForm
from datetime import datetime as dt
from operator import attrgetter
from django.db.models import Q
from itertools import chain
import json
from django.conf import settings
import networkx as nx
import pdb

'''
Employment is the only required fixture.
'''
#using first right now to get around a testing bug where there seems to be a conflict on get between fixtures
EMPLOYMENT = P2Org_Type.objects.filter(relationship_type='employment').first()


def secure(view):
	'''
	Simple decorator that checks SETTINGS.
	'''
	if settings.ROLODEX_SECURE:
		return login_required(view)
	else:
		return view


@secure
def home(request): 
	orgs = Org.objects.all()
	peeps = Person.objects.all()
	peeps = org_relate_peep(peeps)
	return render_to_response('rolodex/home.html',{'orgs':orgs,'peeps':peeps},context_instance=RequestContext(request))


############################################################################
#### ORG Views ####
###################

@secure
def new_org(request): 
	form=OrgForm(instance=Org())
	formset=OrgFormSet(instance=Org())
	if request.method == "POST":
		form = OrgForm(request.POST)
		if form.is_valid():
			org = form.save()
			org.last_edited_by = user_check(request.user)
			org.save()
			formset = OrgFormSet(request.POST,instance=org)
			if formset.is_valid():
				formset.save()
				return redirect('rolodex_org',org.slug)
			else:
				org.delete()
				form = OrgForm(request.POST)
				
	return render_to_response('rolodex/new_org.html',{'form':form,'formset':formset,},context_instance=RequestContext(request))

@secure
def edit_org(request,org_slug):
	orgNODE = Org.objects.get(slug=org_slug)
	form=OrgForm(instance=orgNODE)
	formset=OrgFormSet(instance=orgNODE)
	formset.extra=0
	if request.method == "POST":
		form = OrgForm(request.POST,instance=orgNODE) 
		if form.is_valid():
			org = form.save()
			formset = OrgFormSet(request.POST,instance=org)
			if formset.is_valid():
				formset.save()
				orgNODE.last_edited_by = user_check(request.user)
				orgNODE.save()
				return redirect('rolodex_org',org.slug)

	return render_to_response('rolodex/new_org.html',{'form':form,'formset':formset,'orgNode':orgNODE,'edit':True},context_instance=RequestContext(request))

@permission_required('rolodex.delete_org')
def delete_org(request,org_slug):
	org = Org.objects.get(slug=org_slug)
	if request.method=="POST":
		org.delete()
		return redirect('rolodex_home')
	return render_to_response('rolodex/delete.html',{'org':org},context_instance=RequestContext(request))

@secure
def search_org(request,org_slug):
	node = Org.objects.get(slug=org_slug)
	node.employees = node.get_employees()
	node.contacts = node.org_contact.all()
	node.relations = node.get_relations_with_type()
	node.net_length = len(net_compiler(Org.objects.filter(slug=org_slug),3))
	return render_to_response('rolodex/org.html',{'node':node,},context_instance=RequestContext(request))

@secure
def org_map(request,org_slug):
	node = Org.objects.get(slug=org_slug)
	hops = int(request.GET.get('hops',3))
	return render_to_response('rolodex/orgMap.html',{'node':node,'hops':hops},context_instance=RequestContext(request))

@secure
def org_network(request,org_slug):
	network = net_compiler(Org.objects.filter(slug=org_slug),3)
	data = json.dumps(network)
	return HttpResponse(data, content_type='application/json')

@secure
def adv_org_network(request,org_slug):
	hops = int(request.GET.get('hops',3))
	network = net_compiler(Org.objects.filter(slug=org_slug),hops)
	org = Org.objects.get(slug=org_slug)
	centrality_data = adv_compile(org,hops)
	data = json.dumps({'centrality':centrality_data, 'links':network})
	return HttpResponse(data, content_type='application/json')

@secure
def new_org_relation(request,org_slug):
	saved=False
	org2orgForm = Org2OrgForm()
	org2pForm = Org2PForm()
	if request.method == "POST":
		if request.POST['formType']=='2P':
			org2pForm = Org2PForm(request.POST)
			if org2pForm.is_valid():
				fromEnt = Org.objects.get(pk=request.POST['from_ent'])
				toEnt   = Person.objects.get(pk=request.POST['to_ent'])
				relation= P2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'],)
				fromEnt.add_org2p(toEnt,**{'relation':relation})
				saved=True
		else:	
			org2orgForm = Org2OrgForm(request.POST)
			if org2orgForm.is_valid():
				fromEnt = Org.objects.get(pk=request.POST['from_ent'])
				toEnt   = Org.objects.get(pk=request.POST['to_ent'])
				hierarchy = request.POST['hierarchy']
				relation= Org2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				fromEnt.add_org2org(toEnt,**{'relation':relation,'hierarchy':hierarchy})
				saved=True

	orgs = Org.objects.filter(~Q(slug=org_slug))
	peeps = Person.objects.all()
	peeps = org_relate_peep(peeps)
	orgNode = Org.objects.get(slug=org_slug)
	return render_to_response('rolodex/new_relation.html',{'orgNode':orgNode ,'pForm':org2pForm,'orgForm':org2orgForm,'saved':saved,'peeps':peeps,'orgs':orgs},context_instance=RequestContext(request))


############################################################################
#### PERSON Views ####
######################

@secure
def new_person(request, org_slug=None): 
	form=PersonForm()
	formset=PersonFormSet(instance=Person())
	success = False
	if request.method == "POST":
		form = PersonForm(request.POST) 
		if form.is_valid():
			person = form.save()
			person.last_edited_by = user_check(request.user)
			formset = PersonFormSet(request.POST,instance=person)
			if formset.is_valid():
				formset.save()

				#Add employment relationship
				orgNODE = Org.objects.get(slug=org_slug)
				person.add_p2org(orgNODE,**{'relation':EMPLOYMENT})
				person.save()
			
				form=PersonForm()
				formset=PersonFormSet(instance=Person())
				success = True
			else:
				person.delete()
				form = PersonForm(request.POST)
	
	primary = Org.objects.get(slug=org_slug)
	orgs = Org.objects.all()
	tags = Tag.objects.all()
	return render_to_response('rolodex/new_person.html',{'form':form,'formset':formset,'orgNode':org_slug,'orgs':orgs,'tags':tags,'primary':primary,'success':success},context_instance=RequestContext(request))

@secure
def new_person_no_org(request, org_slug=None): 
	form=PersonForm()
	formset=PersonFormSet(instance=Person())
	success = False
	if request.method == "POST":
		form = PersonForm(request.POST)
		if form.is_valid():
			person = form.save()
			person.last_edited_by = user_check(request.user)
			formset = PersonFormSet(request.POST,instance=person)
			if formset.is_valid():
				formset.save()

				person.save()

				form=PersonForm()
				formset=PersonFormSet(instance=Person())
				success = True
			else:
				person.delete()
				form = PersonForm(request.POST)
	
	orgs = Org.objects.all()
	tags = Tag.objects.all()
	return render_to_response('rolodex/new_person.html',{'form':form,'formset':formset,'orgNode':org_slug,'orgs':orgs,'tags':tags,'primary':None,'success':success},context_instance=RequestContext(request))


@secure
def edit_person(request, person_slug):
	peep = Person.objects.get(slug=person_slug)
	form=PersonForm(instance=peep)
	formset=PersonFormSet(instance=peep)
	formset.extra=0
	if request.method == "POST":
		form = PersonForm(request.POST,instance=peep) 
		
		if form.is_valid():
			person = form.save()
			formset = PersonFormSet(request.POST, instance=person)
			if formset.is_valid():
				formset.save()
				person.last_edited_by = user_check(request.user)
				person.save()
				return redirect('rolodex_person',person.slug)
	
	employer = peep.org_from_p.filter(relation=EMPLOYMENT)
	if len(employer)>0:
		primary = employer[0].to_ent
	else:
		primary = "N/A"	
	orgs = Org.objects.all()
	tags = Tag.objects.all()
	return render_to_response('rolodex/new_person.html',{'form':form,'formset':formset,'personNode':peep,'orgs':orgs,'tags':tags,'primary':primary,'edit':True},context_instance=RequestContext(request))

@permission_required('rolodex.delete_person')
def delete_person(request,person_slug):
	peep = Person.objects.get(slug=person_slug)
	if request.method=="POST":
		peep.delete()
		return redirect('rolodex_home')
	return render_to_response('rolodex/delete.html',{'peep':peep},context_instance=RequestContext(request))

@secure
def search_person(request,person_slug):
	node = Person.objects.get(slug=person_slug)
	employer = node.org_from_p.filter(relation=EMPLOYMENT)
	if len(employer)>0:
		node.primary = employer[0].to_ent.orgName
	else:
		node.primary = "N/A"
	node.contacts = node.person_contact.all()
	node.relations = node.get_relations_with_type()
	node.net_length = len(net_compiler(Person.objects.filter(slug=person_slug),3))
	return render_to_response('rolodex/person.html',{'node':node,},context_instance=RequestContext(request))

@secure
def person_map(request,person_slug):
	node = Person.objects.get(slug=person_slug)
	hops = int(request.GET.get('hops',3))
	return render_to_response('rolodex/personMap.html',{'node':node,'hops':hops},context_instance=RequestContext(request))

@secure
def person_network(request,person_slug):
	network = net_compiler(Person.objects.filter(slug=person_slug),3)
	data = json.dumps(network)
	return HttpResponse(data, content_type='application/json')

@secure
def adv_person_network(request,person_slug):
	hops = int(request.GET.get('hops',3))
	network = net_compiler(Person.objects.filter(slug=person_slug),hops)
	peep = Person.objects.get(slug=person_slug)
	centrality_data = adv_compile(peep,hops)
	data = json.dumps({'centrality':centrality_data, 'links':network})
	return HttpResponse(data, content_type='application/json')

@secure
def new_person_relation(request,person_slug):
	saved=False
	p2orgForm = P2OrgForm()
	p2pForm = P2PForm()
	if request.method == "POST":
		if request.POST['formType']=='2P':
			p2pForm = P2PForm(request.POST)
			if p2pForm.is_valid():
				'''
				We create the relationship via model method rather than the form
				in order to force the symetrical relationship created (so the 
				get_relations method works).
				'''
				fromEnt = Person.objects.get(pk=request.POST['from_ent'])
				toEnt   = Person.objects.get(pk=request.POST['to_ent'])
				relation= P2P_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				fromEnt.add_p2p(toEnt,**{'relation':relation})
				saved=True
		else:	
			p2orgForm = P2OrgForm(request.POST)
			if p2orgForm.is_valid():
				fromEnt = Person.objects.get(pk=request.POST['from_ent'])
				toEnt   = Org.objects.get(pk=request.POST['to_ent'])
				relation= P2Org_Type.objects.get_or_none(relationship_type=request.POST['relation'])
				fromEnt.add_p2org(toEnt,**{'relation':relation})
				saved=True

	orgs = Org.objects.all()
	peeps = Person.objects.filter(~Q(slug=person_slug))
	peeps = org_relate_peep(peeps)
	peepNode = Person.objects.get(slug=person_slug)
	return render_to_response('rolodex/new_relation.html',{'peepNode':peepNode,'pForm':p2pForm,'orgForm':p2orgForm,'saved':saved,'peeps':peeps,'orgs':orgs},context_instance=RequestContext(request))



@secure
def delete_relationship(request):
	'''
	Delete relationships via AJAX on entity pages.
	'''
	if request.POST:
		if request.POST['from_type']=='p':
			if request.POST['to_type'] == 'p':
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_p2p(to_ent)
			else:
				from_ent = Person.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_p2org(to_ent)
		else:
			if request.POST['to_type'] == 'p':
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Person.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_org2p(to_ent)
			else:
				from_ent = Org.objects.get(pk=request.POST['from_ent'])
				to_ent = Org.objects.get(pk=request.POST['to_ent'])
				from_ent.remove_org2org(to_ent)
	return HttpResponse("Done.")



#################################################################
##### GRAPH HELPER FUNCTIONS ######
###################################

def org_relate_peep(peeps):
	'''
	Helper to serialize data for search boxes.
	'''
	for peep in peeps:
		employers = peep.org_relations.filter(p_to_org__relation=EMPLOYMENT)
		if len(employers) > 0:
			peep.orgName=employers[0].orgName
			peep.orgID = employers[0].pk
		else:
			'''
			If person has no primary org relationship, or if it has been deleted, we list
			them with N/A for the purposes of the search boxes on home and relation page.
			'''
			peep.orgName="N/A"
			peep.orgID = "N/A"
	return peeps

def hierarchy(snode,tnode):
	if snode.type == 'org' and tnode.type == 'org':
		snode.hierarchy = Org2Org.objects.get(from_ent=snode.pk,to_ent=tnode.pk).hierarchy
		tnode.hierarchy = Org2Org.objects.get(from_ent=tnode.pk,to_ent=snode.pk).hierarchy
	else:
		snode.hierarchy = 'none'
		tnode.hierarchy = 'none'
	return (snode, tnode)

def get_info(n):
	class output:
		pass
	node = output()
	if n.__class__.__name__ == 'Person':
		node.name = n.firstName +" "+n.lastName
		node.type = "person"
		node.id="p"+str(n.slug)
	else:
		node.name = n.orgName
		node.type = "org"
		node.id="o"+str(n.slug)
	node.pk = n.pk
	return node

def get_relations(nodes):
	relations,node_list=[],[]
	for node in nodes: 
		snode=get_info(node)
		relations = node.get_relations()
		for r in list(chain(relations['orgs'],relations['people'])):
			tnode=get_info(r)
			snode, tnode = hierarchy(snode,tnode)
			node_list.append({ 
				"source":snode.id,
				"source_name":str(snode.name), 
				"source_type":str(snode.type),
				"source_hierarchy":str(snode.hierarchy),
				"target":tnode.id,
				"target_name":str(tnode.name),
				"target_type":str(tnode.type),
				"target_hierarchy":str(tnode.hierarchy),
				})
		relations = list(chain(relations['orgs'],relations['people']))
	relations=list(set(relations))
	return {'node_list':node_list,'relations':relations}

def net_compiler(nodes,hops=2):
	node_list=[]
	for i in range(hops):
		result = get_relations(nodes)
		nodes=result['relations']
		node_list+=result['node_list']
	#de-dup
	node_list=[dict(t) for t in set([tuple(d.items()) for d in node_list])]
	return node_list

def adv_compile(node, hops):
	'''
	Compile some basic network centrality statistics for graph.
	'''
	G = node.nx_graph(hops)
	degree = nx.degree_centrality(G)
	betweenness = nx.betweenness_centrality(G)
	closeness= nx.closeness_centrality(G) 
	data={}
	for node in degree:
		n = get_info(node)
		data[n.id] = {'degree':degree[node],
						'betweenness':betweenness[node],
						'closeness':closeness[node]}
	return data

def user_check(user):
	if user.is_authenticated():
		return user.get_username()
	else:
		return "AnonymousUser"


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, choices=[(b'email', b'email'), (b'phone', b'phone'), (b'link', b'link'), (b'address', b'address')])),
                ('contact', models.CharField(max_length=250, null=True, blank=True)),
                ('notes', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OpenRecordsLaw',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('name', models.CharField(max_length=250)),
                ('link', models.URLField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Org',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('orgName', models.CharField(max_length=200)),
                ('notes', models.TextField(null=True, blank=True)),
                ('openRecordsLaw', models.ForeignKey(blank=True, to='rolodex.OpenRecordsLaw', null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Org2Org',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('from_date', models.DateField(null=True, blank=True)),
                ('to_date', models.DateField(null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('hierarchy', models.CharField(default=b'none', max_length=10, choices=[(b'parent', b'parent'), (b'child', b'child'), (b'none', b'none')])),
                ('from_ent', models.ForeignKey(related_name='org_from_org', to='rolodex.Org')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Org2Org_Type',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('relationship_type', models.CharField(max_length=250)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Org2P',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('from_date', models.DateField(null=True, blank=True)),
                ('to_date', models.DateField(null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('from_ent', models.ForeignKey(related_name='p_from_org', to='rolodex.Org')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrgContactRole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('role', models.CharField(max_length=250)),
                ('description', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='P2Org',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('from_date', models.DateField(null=True, blank=True)),
                ('to_date', models.DateField(null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='P2Org_Type',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('relationship_type', models.CharField(max_length=250)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='P2P',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('from_date', models.DateField(null=True, blank=True)),
                ('to_date', models.DateField(null=True, blank=True)),
                ('description', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='P2P_Type',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('relationship_type', models.CharField(max_length=250)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('lastName', models.CharField(max_length=100)),
                ('firstName', models.CharField(max_length=100)),
                ('position', models.CharField(max_length=250, null=True, blank=True)),
                ('department', models.CharField(max_length=250, null=True, blank=True)),
                ('gender', models.IntegerField(blank=True, null=True, choices=[(1, b'Female'), (2, b'Male'), (3, b'Other')])),
                ('notes', models.TextField(null=True, blank=True)),
                ('org_relations', models.ManyToManyField(related_name='people', through='rolodex.P2Org', to='rolodex.Org', blank=True)),
                ('p_relations', models.ManyToManyField(related_name='+', through='rolodex.P2P', to='rolodex.Person', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonRole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('role', models.CharField(max_length=250)),
                ('description', models.TextField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(unique=True, editable=False)),
                ('tag_name', models.CharField(max_length=250)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='person',
            name='role',
            field=models.ForeignKey(related_name='person_role', blank=True, to='rolodex.PersonRole', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='person',
            name='tags',
            field=models.ManyToManyField(to='rolodex.Tag', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2p',
            name='from_ent',
            field=models.ForeignKey(related_name='p_from_p', to='rolodex.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2p',
            name='relation',
            field=models.ForeignKey(related_name='p2p_relation', blank=True, to='rolodex.P2P_Type', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2p',
            name='to_ent',
            field=models.ForeignKey(related_name='p_to_p', to='rolodex.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2org',
            name='from_ent',
            field=models.ForeignKey(related_name='org_from_p', to='rolodex.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2org',
            name='relation',
            field=models.ForeignKey(related_name='p2org_relation', blank=True, to='rolodex.P2Org_Type', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='p2org',
            name='to_ent',
            field=models.ForeignKey(related_name='p_to_org', to='rolodex.Org'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org2p',
            name='relation',
            field=models.ForeignKey(related_name='org2p_relation', blank=True, to='rolodex.P2Org_Type', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org2p',
            name='to_ent',
            field=models.ForeignKey(related_name='org_to_p', to='rolodex.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org2org',
            name='relation',
            field=models.ForeignKey(related_name='org2org_relation', blank=True, to='rolodex.Org2Org_Type', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org2org',
            name='to_ent',
            field=models.ForeignKey(related_name='org_to_org', to='rolodex.Org'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org',
            name='org_relations',
            field=models.ManyToManyField(related_name='+', through='rolodex.Org2Org', to='rolodex.Org', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org',
            name='p_relations',
            field=models.ManyToManyField(related_name='orgs', through='rolodex.Org2P', to='rolodex.Person', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='org',
            name='tags',
            field=models.ManyToManyField(to='rolodex.Tag', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='contact',
            name='org',
            field=models.ForeignKey(related_name='org_contact', blank=True, editable=False, to='rolodex.Org', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='contact',
            name='person',
            field=models.ForeignKey(related_name='person_contact', blank=True, editable=False, to='rolodex.Person', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='contact',
            name='role',
            field=models.ForeignKey(blank=True, to='rolodex.OrgContactRole', null=True),
            preserve_default=True,
        ),
    ]



from django.db import models
from itertools import chain
from django import forms
from django.core.exceptions import ValidationError
from django.utils.text import slugify
from django.utils.translation import ugettext as _
from django.core.validators import URLValidator,validate_email


from rolodex.slug import unique_slugify

class GetOrNoneManager(models.Manager):
    def get_or_none(self, **kwargs):
        try:
            return self.get(**kwargs)
        except self.model.DoesNotExist:
            return None


###################
## Choice Models ##
###################

'''
Choice models are registered in the admin and are designed to be set by user prior to entering entities into Rolodex.
'''

class OpenRecordsLaw(models.Model):
	'''
	Which open records law applies to the org.
	'''
	slug = models.SlugField(unique=True,editable=False)
	name = models.CharField(max_length=250)
	link = models.URLField(blank=True,null=True)

	def __unicode__(self):
		return self.name

	def save(self, *args, **kwargs):
		unique_slugify(self,self.name)
		super(OpenRecordsLaw, self).save(*args, **kwargs)

class PersonRole(models.Model):
	'''
	Define some roles, preferably ones useful for filtering on, e.g. "Media Contact", "FOIA Officer".
	'''
	slug = models.SlugField(unique=True,editable=False)
	role = models.CharField(max_length=250)
	description = models.TextField(blank=True,null=True)

	def __unicode__(self):
		return self.role

	def save(self, *args, **kwargs):
		unique_slugify(self,self.role)
		super(PersonRole, self).save(*args, **kwargs)

class OrgContactRole(models.Model):
	'''
	Define roles for org contacts, e.g., FOIA email, etc.
	'''
	slug = models.SlugField(unique=True,editable=False)
	role = models.CharField(max_length=250)
	description = models.TextField(blank=True,null=True)

	def __unicode__(self):
		return self.role

	def save(self, *args, **kwargs):
		unique_slugify(self,self.role)
		super(OrgContactRole, self).save(*args, **kwargs)

'''
Relationship types
'''

class P2P_Type(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	relationship_type = models.CharField(max_length=250)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return self.relationship_type

	def save(self, *args, **kwargs):
		unique_slugify(self,self.relationship_type)
		super(P2P_Type, self).save(*args, **kwargs)

class Org2Org_Type(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	relationship_type = models.CharField(max_length=250)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return self.relationship_type

	def save(self, *args, **kwargs):
		unique_slugify(self,self.relationship_type)
		super(Org2Org_Type, self).save(*args, **kwargs)

class P2Org_Type(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	relationship_type = models.CharField(max_length=250)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return self.relationship_type

	def save(self, *args, **kwargs):
		unique_slugify(self,self.relationship_type)
		super(P2Org_Type, self).save(*args, **kwargs)
'''
2.0 feature??? 
'''
class Tag(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	tag_name = models.CharField(max_length=250)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return tag_name

	def save(self, *args, **kwargs):
		unique_slugify(self,self.tag_name)
		super(Tag, self).save(*args, **kwargs)

###################
## Contact Forms ##
###################
contact_types = (('email','email'),('phone','phone'),('link','link'),('address','address'))
class Contact(models.Model):
	'''
	A contact record for persons or orgs.
	'''
	person = models.ForeignKey('Person',blank=True,null=True,editable=False,related_name='person_contact')
	org = models.ForeignKey('Org',blank=True,null=True,editable=False,related_name='org_contact')
	
	type = models.CharField(max_length=100,choices=contact_types)
	contact = models.CharField(max_length=250,blank=True,null=True)
	role = models.ForeignKey('OrgContactRole',blank=True,null=True)
	notes = models.TextField(blank=True,null=True)
	
	def clean(self):
		if not self.person and not self.org:
			raise ValidationError(_('Contact information must be associated with either a person or an organization.'),code='no_ent')
		if self.person and self.org:
			raise ValidationError(_('Contact information should be associated with only one person or organization.'),code='too_many_ents')
		if self.type=='email':
			validate_email(self.contact)
		elif self.type=='link':
			url=URLValidator()
			url(self.contact)
		elif self.type=='phone':
			#Add a validator here?
			pass
		else:
			pass

	def __unicode__(self):
		if self.person:
			return self.person.lastName + ", " + self.person.firstName+": "+self.type
		else:
			return self.org.orgName+": "+self.type

###################
## Entity Models ##
###################

gender_types=((1,'Female'),(2,'Male'),(3,'Other'))

class Person(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	lastName = models.CharField(max_length=100)
	firstName = models.CharField(max_length=100)
	role = models.ForeignKey('PersonRole',blank=True,null=True,related_name='person_role')
	position = models.CharField(max_length=250,blank=True,null=True)
	department = models.CharField(max_length=250,blank=True,null=True)
	gender = models.IntegerField(blank=True,null=True,choices=gender_types)
	#Relationships
	p_relations = models.ManyToManyField('self',through='P2P',symmetrical=False,related_name='+',blank=True)
	org_relations = models.ManyToManyField('Org',through='P2Org',related_name='people',blank=True)

	notes = models.TextField(blank=True, null=True)

	last_edited_by = models.CharField(max_length=250,blank=True,null=True)

	#2.0 feature
	tags = models.ManyToManyField(Tag,blank=True)

	def __unicode__(self):
		return self.lastName+", "+self.firstName

	def save(self, *args, **kwargs):
		unique_slugify(self,self.firstName +"-"+ self.lastName)
		super(Person, self).save(*args, **kwargs)

class Org(models.Model):
	slug = models.SlugField(unique=True,editable=False)
	orgName = models.CharField(max_length=200)
	openRecordsLaw = models.ForeignKey('OpenRecordsLaw',blank=True,null=True)
	#Relationships
	org_relations = models.ManyToManyField('self',through='Org2Org',symmetrical=False,related_name='+',blank=True)
	p_relations = models.ManyToManyField('Person',through="Org2P",related_name='orgs',blank=True)
	
	notes = models.TextField(blank=True, null=True)

	last_edited_by = models.CharField(max_length=250,blank=True,null=True)

	#2.0 feature
	tags = models.ManyToManyField(Tag,blank=True)

	def __unicode__(self):
		return self.orgName

	def save(self, *args, **kwargs):
		unique_slugify(self,self.orgName)
		super(Org, self).save(*args, **kwargs)

###################
## Relationships ##
###################

class P2P(models.Model):
	from_ent = models.ForeignKey(Person, related_name='p_from_p')
	to_ent   = models.ForeignKey(Person,related_name='p_to_p')
	relation = models.ForeignKey('P2P_Type',blank=True, null=True,related_name='p2p_relation')
	from_date = models.DateField(blank=True, null=True)
	to_date = models.DateField(blank=True, null=True)
	description = models.TextField(blank=True, null=True)
	objects = GetOrNoneManager()
	def clean(self):
		if P2P.objects.get_or_none(from_ent=self.from_ent,to_ent=self.to_ent):
			raise ValidationError(_('That relationship already exists.'),code='already_exists')
		super(P2P, self).clean()

	def __unicode__(self):
		return self.from_ent.lastName +" ... "+self.to_ent.lastName
	'''
	Override save and delete methods to maintain relationship symmetry. 
	'''
	def save(self, *args, **kwargs):
		super(P2P, self).save(*args, **kwargs)
		#Enforce Symmetry
		P2P.objects.get_or_create(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)

	def delete(self):
		relation = P2P.objects.get_or_none(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)
		super(P2P, self).delete()
		if relation:
			relation.delete()
		


'''
We make Org2Org relationships unique in that they can have hierarchy. 
Could technically apply to other relationship types, but philosophically
we mean this to convey some principle of ownership. As such doesn't 
really apply to Org2P and definitely not to P2P.
'''
def hierarchy_test(self):
	'''
	Simple 3-way test.
	'''
	if self.hierarchy == 'none':
		return 'none'
	else:
		if self.hierarchy == 'child':
			return 'parent'
		else:
			return 'child'
hierarchy_types = (('parent','parent'),('child','child'),('none','none'))

class Org2Org(models.Model):
	from_ent = models.ForeignKey(Org,related_name='org_from_org')
	to_ent   = models.ForeignKey(Org,related_name='org_to_org')
	relation = models.ForeignKey('Org2Org_Type',blank=True, null=True,related_name='org2org_relation')
	from_date = models.DateField(blank=True, null=True)
	to_date = models.DateField(blank=True, null=True)
	description = models.TextField(blank=True, null=True)
	hierarchy = models.CharField(choices=hierarchy_types, default='none', max_length=10)
	objects = GetOrNoneManager()
	
	def __unicode__(self):
		return self.from_ent.orgName +" ... "+self.to_ent.orgName
	
	def clean(self):
		if Org2Org.objects.get_or_none(from_ent=self.from_ent,to_ent=self.to_ent):
			raise ValidationError(_('That relationship already exists.'),code='already_exists')
		super(Org2Org, self).clean()
	
	def save(self, *args, **kwargs):
		super(Org2Org, self).save(*args, **kwargs)
		'''
		Enforce Symmetry.
		An issue: If you change a feature of a relationship, e.g., hierarchy, you'll end up creating a new relationship, and the stale one will persist.
		'''
		relation, got = Org2Org.objects.get_or_create(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description,
			hierarchy=hierarchy_test(self))
	
	def delete(self,*args,**kwargs):
		#Enforce Symmetry
		relation = Org2Org.objects.get_or_none(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description )
		super(Org2Org, self).delete(*args, **kwargs)
		if relation:
			relation.delete()


class P2Org(models.Model):
	from_ent = models.ForeignKey(Person,related_name='org_from_p')
	to_ent   = models.ForeignKey(Org,related_name='p_to_org')
	relation = models.ForeignKey('P2Org_Type',blank=True, null=True,related_name='p2org_relation')
	from_date = models.DateField(blank=True,null=True)
	to_date = models.DateField(blank=True,null=True)
	description = models.TextField(blank=True,null=True)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return self.from_ent.lastName +" ... "+self.to_ent.orgName

	def clean(self):
		if P2Org.objects.get_or_none(from_ent=self.from_ent,to_ent=self.to_ent):
			raise ValidationError(_('That relationship already exists.'),code='already_exists')
		super(P2Org, self).clean()
	
	def save(self, *args, **kwargs):
		super(P2Org, self).save(*args, **kwargs)
		#Enforce Symmetry
		Org2P.objects.get_or_create(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)
	
	def delete(self,*args,**kwargs):
		#Enforce Symmetry
		relation = Org2P.objects.get_or_none(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)
		super(P2Org, self).delete(*args, **kwargs)
		if relation:
			relation.delete()

class Org2P(models.Model):
	from_ent = models.ForeignKey(Org, related_name='p_from_org')
	to_ent   = models.ForeignKey(Person,related_name='org_to_p')
	relation = models.ForeignKey('P2Org_Type',blank=True, null=True,related_name='org2p_relation')
	from_date = models.DateField(blank=True,null=True)
	to_date = models.DateField(blank=True,null=True)
	description = models.TextField(blank=True,null=True)
	objects = GetOrNoneManager()

	def __unicode__(self):
		return self.from_ent.orgName +" ... "+self.to_ent.lastName

	def clean(self):
		if Org2P.objects.get_or_none(from_ent=self.from_ent,to_ent=self.to_ent):
			raise ValidationError(_('That relationship already exists.'),code='already_exists')
		super(Org2P, self).clean()

	def save(self, *args, **kwargs):
		super(Org2P, self).save(*args, **kwargs)
		#Enforce Symmetry
		P2Org.objects.get_or_create(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)

	def delete(self,*args,**kwargs):
		#Enforce Symmetry
		relation = P2Org.objects.get_or_none(
			from_ent=self.to_ent,
			to_ent=self.from_ent,
			relation=self.relation,
			from_date=self.from_date,
			to_date=self.to_date,
			description=self.description)
		super(Org2P, self).delete(*args, **kwargs)
		if relation:
			relation.delete()

##################
## Setter Funcs ##
##################

def add_p2p(self, person,symm=True,**kwargs):
    relationship = P2P.objects.get_or_create(
        from_ent=self,
        to_ent=person,
        **kwargs)
    return relationship
def remove_p2p(self, person, symm=True,**kwargs):
	relation = P2P.objects.get_or_none(
        from_ent=self, 
        to_ent=person,
        **kwargs)
	if relation:
		relation.delete()
def add_org2org(self,org,symm=True,**kwargs):
	relationship = Org2Org.objects.get_or_create(
		from_ent=self,
		to_ent=org,
		**kwargs)
	return relationship
def remove_org2org(self,org,symm=True,**kwargs):
	relation = Org2Org.objects.get_or_none(
        from_ent=self, 
        to_ent=org,
        **kwargs)
	if relation:
		relation.delete()
def add_p2org(self,org,symm=True,**kwargs):
	relationship = P2Org.objects.get_or_create(
		from_ent=self,
		to_ent=org,
		**kwargs)
	return relationship
def remove_p2org(self,org,symm=True,**kwargs):
	relation = P2Org.objects.get_or_none(
        from_ent=self, 
        to_ent=org,
        **kwargs)
	if relation:
		relation.delete()
def add_org2p(self,person,symm=True,**kwargs):
	relationship = Org2P.objects.get_or_create(
		from_ent=self,
		to_ent=person,
		**kwargs)
	return relationship
def remove_org2p(self,person,symm=True,**kwargs):
	relation = Org2P.objects.get_or_none(
        from_ent=self, 
        to_ent=person,
        **kwargs)
	if relation:
		relation.delete()


##################
## Getter Funcs ##
##################

def get_relations(self):
	people=self.p_relations.all()
	orgs = self.org_relations.all()
	return {'people':people,'orgs':orgs}

def get_employer(self):
	relates=self.org_from_p.filter(relation__relationship_type='employment')
	employers=[r.to_ent for r in relates]
	return employers

def get_employees(self):
	relates=self.p_to_org.filter(relation__relationship_type='employment')
	employees=[r.from_ent for r in relates]
	return employees

def get_employees_by_role(self,role):
	'''
	Thinking preferable to pass a string? Could also refactor for role object.
	'''
	relates=self.p_to_org.filter(relation__relationship_type='employment',from_ent__role__role=role)
	employees=[r.from_ent for r in relates]
	return employees

def get_relations_by_type(self,type):
	'''
	Preferable to pass a string? Could also refactor for relation object.
	'''
	if self._meta.model_name == 'org':
		kith=self.p_to_org.filter(relation__relationship_type=type)
		kin=self.org_to_org.filter(relation__relationship_type=type)
		return {'people':collate_relations(kith),'orgs':collate_relations(kin)}
	else: # =='person'
		kith=self.org_to_p.filter(relation__relationship_type=type)
		kin=self.p_to_p.filter(relation__relationship_type=type)
		return {'orgs':collate_relations(kith),'people':collate_relations(kin)}

def get_relations_with_type(self):
	if self._meta.model_name == 'org':
		kith=self.p_to_org.all().order_by('id')
		kin=self.org_to_org.all().order_by('id')
		return {'people':collate_relations(kith),'orgs':collate_relations(kin)}
	else: # =='person'
		kith=self.org_to_p.all().order_by('id')
		kin=self.p_to_p.all().order_by('id')
		return {'orgs':collate_relations(kith),'people':collate_relations(kin)}

'''
hierarchy getters for orgs
'''
def get_children(self):
	orgs = self.org_to_org.filter(hierarchy='child')
	return [org.from_ent for org in orgs]

def get_parents(self):
	orgs = self.org_to_org.filter(hierarchy='parent')
	return [org.from_ent for org in orgs]


###################
## Network Funcs ##
###################

def collate_relations(queryset):
	'''
	Presumes self == to_ent
	'''
	relation_list=[]
	for q in queryset:
		if q.relation:
			relation_list.append({'type':q.relation.relationship_type,'relation':q.from_ent})
		else:
			relation_list.append({'type':'','relation':q.from_ent})
	return relation_list

import networkx as nx
def nx_add(graph,node):
	people = node.p_relations.all()
	orgs = node.org_relations.all()
	ents = list(chain(people,orgs))
	graph.add_nodes_from(ents)
	graph.add_edges_from([(node,n) for n in ents])
	return graph

def relation_list(self):
	return list( chain( self.p_relations.all(),self.org_relations.all() ) )

def nx_graph(self,hops=2):
	G = nx.Graph()
	G = nx_add(G,self)
	relations = relation_list(self)
	for i in range(hops-1):
		intermediate = []
		for r in relations:
			G = nx_add(G,r)
			intermediate = list( chain( intermediate,relation_list(r) ) ) 
		relations = list( chain( relations,intermediate ) )
	return G


###############
## Add funcs ##
###############

Person.add_p2p = add_p2p
Person.remove_p2p = remove_p2p
Person.add_p2org = add_p2org
Person.remove_p2org = remove_p2org
Person.get_relations = get_relations
Person.get_employer = get_employer
Person.get_relations_with_type = get_relations_with_type
Person.get_relations_by_type = get_relations_by_type
Person.nx_graph = nx_graph

Org.add_org2org = add_org2org
Org.remove_org2org = remove_org2org
Org.add_org2p = add_org2p
Org.remove_org2p = remove_org2p
Org.get_relations = get_relations
Org.get_employees = get_employees
Org.get_employees_by_role = get_employees_by_role
Org.get_relations_with_type = get_relations_with_type
Org.get_relations_by_type = get_relations_by_type
Org.get_parents = get_parents
Org.get_children = get_children
Org.nx_graph = nx_graph


class RolodexRouter(object):
	def db_for_read(self, model, **hints):
		if model._meta.app_label == 'rolodex':
			return 'rolodex'
		else:
			return 'default'
	def db_for_write(self, model, **hints):
		if model._meta.app_label == 'rolodex':
			return 'rolodex'
		else:
			return 'default'
	def allow_relation(self, obj1, obj2, **hints):
		if obj1._meta.app_label == 'rolodex' or obj2._meta.app_label == 'rolodex':
			return True
		return None

	def allow_migrate(self, db, model):
		if db == 'rolodex':
			return model._meta.app_label == 'rolodex'
		elif model._meta.app_label == 'rolodex':
			return False
		return None


from django.conf.urls import patterns, url, include
from rolodex import views, API
from rolodex.API import router, views as APIviews



urlpatterns = patterns('',
    url(r'^$', views.home, name='rolodex_home'),
    url(r'^org/(.+)/$',views.search_org,name='rolodex_org'),
    url(r'^person/(.+)/$',views.search_person,name='rolodex_person'),

    #Entity maintenance
    url(r'^add-person/(.+)/$',views.new_person,name='rolodex_new_person'),
    url(r'^add-person/$',views.new_person_no_org,name='rolodex_new_person_no_org'),
    url(r'^edit-person/(.+)/$',views.edit_person,name='rolodex_edit_person'),
    url(r'^delete-person/(.+)/$',views.delete_person,name='rolodex_delete_person'),
    url(r'^add-org/$',views.new_org,name='rolodex_new_org'),
    url(r'^edit-org/(.+)/$',views.edit_org,name='rolodex_edit_org'),
    url(r'^delete-org/(.+)/$',views.delete_org,name='rolodex_delete_org'),
    
    #Relationship maintenance
    url(r'^add-relationship/org/(.+)/$',views.new_org_relation,name='rolodex_new_org_relation'),
    url(r'^add-relationship/person/(.+)/$',views.new_person_relation,name='rolodex_new_person_relation'),
    url(r'^delete-relationship/$',views.delete_relationship,name='rolodex_delete_relation'),


    #network graphs
    url(r'^person-map/(.+)/$',views.person_map,name='rolodex_person_map'),
    url(r'^org-map/(.+)/$',views.org_map,name='rolodex_org_map'),
    url(r'^person-network/(.+)/$',views.person_network,name='rolodex_person_network'),
    url(r'^org-network/(.+)/$',views.org_network,name='rolodex_org_network'),
    url(r'^person-network-advanced/person/(.+)/$',views.adv_person_network,name='rolodex_adv_person_network'),
    url(r'^org-network-advanced/org/(.+)/$',views.adv_org_network,name='rolodex_adv_org_network'),
    
    #api
    url(r'^api/',include(router.router.urls)),
    url(r'^api/employees/([0-9]+)/$',APIviews.GetEmployees.as_view())
)

'''
    Work on more API calls to match model methods (though some model funcs 
        are passed strings and others objects...)
'''


from rolodex.models import Org, Person, P2P,Org2Org,P2Org,Org2P,Contact
from django import forms
from django.forms import ModelForm
from django.forms.models import inlineformset_factory



class OrgForm(ModelForm):
    class Meta:
        model = Org
        fields = ['orgName','openRecordsLaw','notes']
        widgets = {
        			'orgName': forms.TextInput(attrs={'class':'form-control','placeholder':'Name'}),
        			'openRecordsLaw': forms.Select(attrs={'class':'form-control'}),
        			'notes': forms.Textarea(attrs={'class':'form-control'}),
        		}

class PersonForm(ModelForm):
	class Meta:
		model = Person
		fields = ['lastName','firstName','role','position','department','gender','notes']
		widgets = {
					'lastName': forms.TextInput(attrs={'class':'form-control','placeholder':'Last name'}),
					'firstName': forms.TextInput(attrs={'class':'form-control', 'placeholder':'First name'}),
					'position': forms.TextInput(attrs={'class':'form-control'}),
					'department': forms.TextInput(attrs={'class':'form-control'}),
					'gender': forms.Select(attrs={'class':'form-control'}),
					'role': forms.Select(attrs={'class':'form-control'}),
					'notes': forms.Textarea(attrs={'class':'form-control'}),
				}

PersonFormSet = inlineformset_factory(Person,Contact,extra=1,can_delete=True,
	exclude=(),
	widgets = {
		  'contact': forms.TextInput(attrs={'class':'form-control','placeholder':'Enter contact'}),
          'notes': forms.Textarea(attrs={'rows':5, 'cols':35, 'class':'form-control','placeholder':'Notes'}),
          'type': forms.Select(attrs={'class':'form-control'}),
        })

OrgFormSet = inlineformset_factory(Org,Contact,extra=1,can_delete=True,
	exclude=(),
	widgets = {
		  'contact': forms.TextInput(attrs={'class':'form-control','placeholder':'Enter contact'}),
          'notes': forms.Textarea(attrs={'rows':5, 'cols':35, 'class':'form-control','placeholder':'Notes'}),
          'type': forms.Select(attrs={'class':'form-control'}),
          'role': forms.Select(attrs={'class':'form-control role'}),
        })
########################
## Relationship Forms ##
########################
class P2PForm(ModelForm):
	class Meta:
		model = P2P
		fields = ['from_ent','to_ent','relation','from_date','to_date','description']
		widgets = {'relation': forms.Select(attrs={'class':'form-control'}),}
class P2OrgForm(ModelForm):
	class Meta:
		model = P2Org
		fields = ['from_ent','to_ent','relation','from_date','to_date','description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}),}
class Org2PForm(ModelForm):
	class Meta:
		model = Org2P
		fields = ['from_ent','to_ent','relation','from_date','to_date','description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}),}
class Org2OrgForm(ModelForm):
	class Meta:
		model = Org2Org
		fields = ['from_ent','to_ent','relation','from_date','to_date','hierarchy','description']
		widgets = {'relation': forms.Select(attrs={'class': 'form-control'}),
					'hierarchy':forms.Select(attrs={'class':'form-control'})}


from rest_framework import serializers

from rolodex.models import Person,Org,Contact,PersonRole,OrgContactRole,P2P,Org2Org,P2Org,Org2P,P2P_Type,Org2Org_Type,P2Org_Type
from django.contrib.auth.models import User



class RoleSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = PersonRole
		fields = ('role','description')

class ContactRoleSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = OrgContactRole
		fields = ('role','description')


class P2P_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = P2P_Type 
		fields = ('relationship_type',)
class Org2Org_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = Org2Org_Type 
		fields = ('relationship_type',)
class P2Org_TypeSerializer(serializers.HyperlinkedModelSerializer):
	class Meta:
		model = P2Org_Type 
		fields = ('relationship_type',)


class PersonSerializer(serializers.HyperlinkedModelSerializer):
	# people = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='p_relations')
	# orgs = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='org_relations')
	# person_role = serializers.HyperlinkedRelatedField(many=True, read_only=False,view_name='role',queryset=PersonRole.objects.all())
	class Meta:
		model = Person
		fields = ('slug','firstName', 'lastName','position','department','gender','role','person_contact','org_relations','p_relations')

class OrgSerializer(serializers.HyperlinkedModelSerializer):
	# people = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='person-relations')
	# orgs = serializers.HyperlinkedRelatedField(many=True, read_only=True,view_name='org-relations')
	class Meta:
		model = Org
		fields = ('slug','orgName','org_relations','p_relations','org_contact')

class ContactSerializer(serializers.HyperlinkedModelSerializer):
	# person_contact = serializers.HyperlinkedRelatedField( read_only=False,view_name='person',queryset=Person.objects.all())
	# org_contact = serializers.HyperlinkedRelatedField( read_only=False,view_name='org',queryset=Org.objects.all())
	class Meta:
		model = Contact
		fields = ('id','person','org','type','contact','role','notes')

class P2PSerializer(serializers.HyperlinkedModelSerializer):
	# p_from_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_person',queryset=Person.objects.all())
	# p_to_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_person',queryset=Person.objects.all())
	# p2p_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2P_Type.objects.all())
	class Meta:
		model = P2P
		fields = ('id','from_ent','to_ent','relation','from_date','to_date','description')

class Org2OrgSerializer(serializers.HyperlinkedModelSerializer):
	# org_from_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_org',queryset=Org.objects.all())
	# org_to_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_org',queryset=Org.objects.all())
	# org2org_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=Org2Org_Type.objects.all())
	class Meta:
		model = Org2Org
		fields = ('id','from_ent','to_ent','relation','from_date','to_date','description')

class P2OrgSerializer(serializers.HyperlinkedModelSerializer):
	# org_from_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_person',queryset=Person.objects.all())
	# p_to_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_org',queryset=Org.objects.all())
	# p2org_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2Org_Type.objects.all())
	class Meta:
		model = P2Org
		fields = ('id','from_ent','to_ent','relation','from_date','to_date','description')

class Org2PSerializer(serializers.HyperlinkedModelSerializer):
	# p_from_org = serializers.HyperlinkedRelatedField( read_only=False,view_name='from_org',queryset=Org.objects.all())
	# org_to_p = serializers.HyperlinkedRelatedField( read_only=False,view_name='to_person',queryset=Person.objects.all())
	# org2p_relation = serializers.HyperlinkedRelatedField( read_only=False,view_name='relationship_type',queryset=P2Org_Type.objects.all())
	class Meta:
		model = Org2P
		fields = ('id','from_ent','to_ent','relation','from_date','to_date','description')


import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-rolodex',
    version='0.1.2.5',
    packages=['rolodex'],
    include_package_data=True,
    license='MIT License',
    description='A Django app for managing relationships between people and organizations.',
    long_description=README,
    url='https://github.com/DallasMorningNews/django-rolodex/',
    author='Jon McClure',
    author_email='jon.r.mcclure@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires = ['Django>=1.7','djangorestframework>=2.4.4', 'dj-database-url>=0.3.0','networkx>=1.9.1',],
)




from rolodex.models import Person, Org
from rolodex.views import org_relate_peep
def modal_context(request):
	modal_peeps = Person.objects.all()
	modal_orgs = Org.objects.all()
	modal_peeps = org_relate_peep(modal_peeps)
	return {'modal_peeps':modal_peeps,'modal_orgs':modal_orgs,}